#ifndef __SCS_PROCESS_H__
#define __SCS_PROCESS_H__

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>
#include <time.h>
#include <dirent.h>
#include <signal.h>
#include <pthread.h>

#include <sys/stat.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <net/if.h>

#define SCS_CLIENT_MAX 128

#define ARCH_MIPS64EL 0
#define ARCH_X86_64 1
#define ARCH_AARCH64 2

typedef struct scs_client_list
{
	char client_ip[SCS_CLIENT_MAX];
	struct scs_client_list *next;
}scs_client;

typedef struct scs_message_list
{
    char *msg;
    int msg_len;
    struct scs_message_list *next;
}scs_message;

const char *repo_path_mips64el;
const char *repo_path_x86_64;
const char *repo_path_aarch64;
const char *data_file_path_mips64el;
const char *data_file_path_x86_64;
const char *data_file_path_aarch64;
const char *push_file_path_mips64el;
const char *push_file_path_x86_64;
const char *push_file_path_aarch64;

scs_client *scs_list_mips64el;
scs_client *scs_list_x86_64;
scs_client *scs_list_aarch64;
pthread_mutex_t client_list_mutex_mips64el;
pthread_mutex_t client_list_mutex_x86_64;
pthread_mutex_t client_list_mutex_aarch64;

scs_message *scs_message_queue_mips64el;
scs_message *scs_message_queue_x86_64;
scs_message *scs_message_queue_aarch64;
pthread_mutex_t message_queue_mutex_mips64el;
pthread_mutex_t message_queue_mutex_x86_64;
pthread_mutex_t message_queue_mutex_aarch64;

int scs_list_init(scs_client **scs_list);
int scs_list_add(scs_client *scs_list, char *client_ip);
int scs_list_remove(scs_client *scs_list, char *client_ip);

int scs_store_message(scs_message *scs_list, char *msg, int msg_len);
int scs_restore_and_send_message(int arch);
int scs_message_broadcast(char *requ_msg, int requ_len);

int scs_generate_data_file(int arch);
int scs_generate_push_file(int arch, char *msg, int msg_len);

int scs_process_handler(char *requ_msg, int requ_len);

#endif
